#!/usr/bin/python
# -*- coding: UTF-8 -*-
#
# 21.09.2014

import tkinter as tk
from tkinter import messagebox
from tkinter import filedialog
from tkinter import scrolledtext
from io import StringIO
import os
import my_dialogs

APP_NAME = "Editor"


class Editor(tk.Toplevel):
    """
    Dem Editor wird übergeben:
    -   ein gültiger Dateiname, die Datei wird dann eingelesen
    oder
    -   ein Textstring oder
    -   ein StringIO
    """
    # Beim Öffnen von Dateien werden diese Endodings in der vorliegenden
    # Reihenfolge durchprobiert
    # Beim Speichern wird dieses Encoding verwendet.
    encodings = ("Latin-1", "UTF-8")

    def __init__(self, master, text = None, file = None, height = 60, protect = 0, **kwargs):

        tk.Toplevel.__init__(self, master)
        self.protocol('WM_DELETE_WINDOW', self.onAppClose)
        self.appName = APP_NAME

        wpos = master.winfo_geometry().split('+')
        self.geometry("+{}+{}".format(int(wpos[1])+ 100, int(wpos[2]) + 100))
        self.resizable(width = None, height = None)

        self.file = file
        self.text = text
        self.height = height

        # für Suchen und Ersetzen
        self.searchStr  = ""
        self.replaceStr = ""
        self.seekAll    = False
        self.seekReg    = False
        # Vorgabe für das Encoding neuer Dateien
        # Das Encoding von neu geöffneten Dateien wird in der Reiehnfolge der
        # Einträge in Editor.encodings[] ausprobiert.
        self.encoding   = Editor.encodings[0]

        # self.protect muss vor addMenus definiert werden
        self.protect = tk.IntVar()

        self.addMenus()
        self.addWidgets()
        self.addBindings()

        # Den Editor mit Text füllen
        if text is not None:
                self.text = text
                self._putText()

        elif file is not None:
            if type(file) is StringIO:
                self.text = file.getvalue()
                self._putText()

            else:
                self.readFile()

        self.protect.set(protect)          # ohne Scheibschutz als Vorgabe
        self.appTitle()

        if kwargs:
            try:
                self.searchStr  = kwargs['searchStr']
                self.replaceStr = kwargs['replaceStr']
                self.seekReplace = kwargs['replace']
                self.seekCase   = kwargs['seekCase']
                self.on_replace = kwargs['replace']
                self.seekAll    = kwargs['seekAll']
                self.seekReg    = kwargs['regex']

            except KeyError:
                print("Keyerror")
                return
            self.after(100, self.onSeek(False))

        self.transient(master)              # 'self' im Vordergrund vor "master" halten
        self.grab_set()                     # events gehen nur an diesen Dialog
        #master.wait_window(self)            # Das übergeordnete Fenster wartet

        self.mainloop()                                    # bis 'self' geschlossen wird
        self.destroy()

    def addMenus(self):
        m = tk.Menu(self)
        self['menu'] = m
        #self.configure(menu = m)

        datei = tk.Menu(master = m, tearoff = False)

        datei.add_command(label = "Datei neu ...", command = self.onNew, underline = 0)
        datei.add_command(label = "Datei öffnen ...", command = self.onOpen, underline = 6)

        datei.add_separator()

        datei.add_command(label = "Speichern unter ...", command = self.onSaveas, underline = 10)
        datei.add_command(label = "Speichern ", command = self.onSave, underline = 0)

        datei.add_separator()

        datei.add_checkbutton(label = "Schreibschutz", variable = self.protect,
                                onvalue = 1, offvalue = 0, command = self.onProtect, underline = 3)
        datei.add_separator()
        datei.add_command(label = "Beenden", command = self.onAppClose, underline = 0)

        m.add_cascade(label = "Datei", menu = datei, underline = 0)

        edit = tk.Menu(master = m, tearoff = False)
        edit.add_command(label = "Undo", command = self.onUndo, underline = 0)
        edit.add_command(label = "Redo", command = self.onRedo, underline = 0)

        edit.add_separator()

        edit.add_command(label = "Kopieren",   command = self.onCopy, underline = 0)
        edit.add_command(label = "Ausschneiden",   command = self.onCut, underline = 0)
        edit.add_separator()
        edit.add_command(label = "Einfügen",   command = self.onInsert, underline = 0)
        edit.add_separator()
        edit.add_command(label = "Suchen / Ersetzen [^F3]",   command = self.onSeek, underline = 0)
        edit.add_command(label = "Weitersuchen [F3]",   command = self.onSeeknext, underline = 0)
        edit.add_separator()
        edit.add_command(label = "goto Line [F4]",   command = self.onGoto, underline = 0)
        m.add_cascade(label = "Bearbeiten", menu = edit, underline = 0)


    def addBindings(self):
        self.t.bind('<F3>', self.onSeeknext)
        self.t.bind('<Control-F3>', self.onSeek)
        self.t.bind('<F4>', self.onGoto)
        self.t.bind('<Escape>', self.onAppClose)


    def addWidgets(self):
        f = tk.LabelFrame(self)
        f.grid(column = 0, row = 0)
        self.t = scrolledtext.ScrolledText(f, width = 120, height = self.height)
        self.t.grid(column = 0, row = 0)
        self.t.insert(0.0, " ")  # einen Platzhalter den Editor einfügen
        self.t.mark_set(tk.INSERT, '1.0')
        self.t.see(tk.INSERT)
        self.t.focus_set()


    def onAppClose(self, e = None):
        """ Das Programm beenden,
            wenn Ändreungen vorgenommen worden sind, dann nachfragen
            aber nicht automatisch speichern.
        """
        if self.t.edit_modified():
            if messagebox.askokcancel(self.appName,
            "Die Datei wurde geändert !\nÄnderungen verwerfen ?"):
                self.quit()

            else:
                # hier ggf. Optionen zum Speichern als einsetzen
                pass
        else:
            self.quit()


    def onCopy(self):
        """ Den gesamten Inhalt des Editorfensters ins Clipboard kopieren """
        if not self.t.tag_ranges(tk.SEL):
            messagebox.showwarning(self.appName, "Kein Text ausgewählt !")
        else:
            self.clipboard_clear()
        #self.clipboard_append(self.t.get(0.0, 'end'))
            self.clipboard_append(self.t.get(tk.SEL_FIRST, tk.SEL_LAST))


    def onCut(self):
        if not self.t.tag_ranges(tk.SEL):
            messagebox.showwarning(self.appName, "Kein Text ausgewählt !")
        else:
            self.clipboard_clear()
            self.clipboard_append(self.t.get(tk.SEL_FIRST, tk.SEL_LAST))
            self.t.delete(tk.SEL_FIRST, tk.SEL_LAST)


    def onInsert(self):
        try:
            text = self.t.selection_get(selection = 'CLIPBOARD')

        except tk.TclError:
            messagebox.showwarning(self.appName, "Das Clipboard ist leer !")
            return
        self.t.insert(tk.INSERT, text)


    def onUndo(self):
        try:
            self.t.edit_undo()
        except tk.TclError:
            messagebox.showwarning(self.appName, "Kein Undo verfügbar !")


    def onRedo(self):
        try:
            self.t.edit_redo()
        except tk.TclError:
            messagebox.showwarning(self.appName, "Kein Redo verfügbar !")


    def onSeeknext(self, e = None):
        if not self.searchStr:
            tk.messagebox.showwarning(self.appName, "Kein Suchstring definiert")
        else:
            self.onSeek(newsearch = False)


    def onSeek(self, newsearch = True):
        # bestehende Auswahl löschen
        self.t.tag_remove(tk.SEL, '1.0', tk.END)

        # wenn nicht über "weitersuchen" aufgerufen wird, dann die Suchparameter
        # im Dialog abfragen und an den Anfang des Dokumentes springen
        if newsearch:
            askSearch(self, self.askSearch_callback, title = self.appName)
            # bestehende Selektion löschen und an den Anfang der Datei gehen
            self.t.tag_remove(tk.SEL, '1.0', tk.END)
            self.t.mark_set(tk.INSERT, '1.0')

        # kommentarlos beenden, wenn kein Suchstring definiert ist
        if not self.searchStr:
            return

        self.t.focus_set()

        self.cnt_found = 0
        self.cnt_replaced = 0

        if self.seekReplace:
            if not self.replaceStr:
                self.seekReplace = tk.messagebox.askokcancel(self.appName,
                    'Den Suchstring "{}" löschen ?'.format(self.searchStr))

        # solange suchen, wie die Suche erfolgreich ist,
        # nur wenn seekAll == False, dann wird nur eine einzige Suche durchgeführt
        where = "?"
        length = tk.IntVar()
        while self.searchStr and where:
            where = self.t.search(self.searchStr, tk.INSERT, tk.END,
                    #exact = not self.seekReg,
                    count = length,
                    nocase = self.seekCase,
                    regexp = self.seekReg)

            if where:
                #last = "{}+{}c".format(where, len(self.searchStr))
                last = "{}+{}c".format(where, length.get())

                self.cnt_found += 1

                # hier wird ggf. ersetzt und last neu berechnet
                if self.seekReplace: #and self.replaceStr:
                    self.t.delete(where, last)
                    self.t.insert(where, self.replaceStr)
                    last = "{}+{}c".format(where, len(self.replaceStr))
                    self.cnt_replaced += 1

                self.t.tag_add(tk.SEL, where, last)
                #self.t.tag_add(tk.SEL, tk.SEL_FIRST, tk.SEL_LAST)
                self.t.mark_set(tk.INSERT, last)
                self.t.see(tk.INSERT)
#                self.t.update()

            if not self.seekAll:
                break

        if self.seekAll or self.cnt_found == 0:
            my_dialogs.showinfo(self, self.appName,
                    "{} Fundstellen,\n{} Ersetzungen".\
                    format(self.cnt_found, self.cnt_replaced))

# Diese Variante bringt eine falsche Reihenfolge der Windows
# Das Root-Window popt vor den Editor ..
# Erst nach dem Schließen von showinf0() stimmt die Folge wieder

##            tk.messagebox.showinfo(self.appName,
##                    "{} Fundstellen,\n{} Ersetzungen".\
##                    format(self.cnt_found, self.cnt_replaced))

            self.t.focus_force()


    def onProtect(self):
        """ Die Checkbox für den Schreibschutz wurde geändert
            In der Checkbox wurde der angezeigte Wert invertiert, also bedeutet:
                0 -> der Schreibschutz soll deaktiviert werden
                1 -> der Schreibschutz soll aktiviert werden
        """
        if self.protect.get() :
            self.t.configure(state = tk.DISABLED)
            self.protect.set(1)
            messagebox.showinfo(self.appName, "Der Schreibschutz wurde wieder eingerichtet")

        else:
            if messagebox.askyesno(self.appName, "Den Schreibschutz aufheben ?"):
                self.t.configure(state = tk.NORMAL)
                self.protect.set(0)
            else:
                self.protect.set(1)     # wieder aktivieren, da der Checkbutton
                                        # den Wert verändert hat
        self.appTitle()


    def onNew(self):
        if self.t.edit_modified():
            if not messagebox.askyesno(self.appName, \
                    "Datei wurde geändert\nÄnderungen verwerfen ?"):
                return
                #self.file_save()

        self.text = ""
        self.protect.set(0)
        self._putText()


    def onOpen(self):
        """ Wird aus dem Menue heraus aufgerufen """
        if self.t.edit_modified():
            if not messagebox.askyesno(self.appName, \
                    "Datei wurde geändert\nÄnderungen verwerfen ?"):
                return


        self.file = filedialog.askopenfilename(defaultextension = '.txt',\
                                title = "Datei öffnen",\
                                initialfile = "file",\
                                filetypes =[("TXT", "*.txt")] )
        # den Schreibschutz aufheben
        self.protect.set(0)
        self.readFile()


    def readFile(self, args = None):
        """ öffnet eine Datei, testet verschiedene Encodings beim Einlesen """

        if args is None and self.file is not None:
            file = self.file

        else:
            file = args[0]
            print("readfile : ", file)
            if not os.path.isfile(file):
                tk.messagebox.showwarning(self.appName, "Datei {} ist nicht vorhanden !")
                return

        for encoding in Editor.encodings:
            try:
                f = open(file, "r", encoding = encoding)
                self.text = f.read()
                f.close()
                self.encoding = encoding

            except UnicodeError:
                print ("Error in readFile() @ Encoding: {}".format(encoding))

        self._putText()


    def _putText(self):
        """ Fügt einen Text in den Editor ein,
            der bestehende Text im Editor wird dabei überschrieben
        """
        if self.protect.get():
            self.t.config(state = tk.DISABLED)
        else:
            self.t.config(state = tk.NORMAL)

        self.t.delete('1.0', tk.END)
        self.t.insert('1.0', self.text)

        self.clipboard_clear()
        self.t.edit_modified(False)
        self.t.edit_reset()
        self.t['undo'] = True
        self.t['autoseparators'] = True
        self.t.edit_separator()

        self.t.mark_set(tk.INSERT, '1.0')
        self.t.see(tk.INSERT)
        self.t.focus_set()

        self.appTitle()


    def appTitle(self):
        """
        Passt den Titel des Fensters an:
        Ausgabe des Filetyps, des Filenamens und ggf. readonly
        """
        if self.protect.get():
            protect = "[readonly]"
        else:
            protect = ""

        if self.file is not None:
            if type(self.file) is StringIO:
                self.title("{} : StringIO {}".format(self.appName, protect))
            else:
                self.title("{} : {} {}".format(self.appName, self.file, protect))
        else:
            self.title("{} : no file : {}".format(self.appName, protect))


    def onSaveas(self):
        """ Der Menüpunkt 'Speichern unter' wurde aufgerufen """
        self.file = filedialog.asksaveasfilename(defaultextension = '.txt',\
                                    title = "Datei speichern",\
                                    initialfile = self.file,\
                                    filetypes = [("TXT", "*.txt"),
                                                 ("PY", "*.py"),
                                                 ("alle Dateien", "*.*") ] )
        if self.file is not None:
            try:
                f = open(self.file, "w", encoding = self.encoding)
                f.write(self.t.get('1.0', 'end'))
                f.close()
                self.t.edit_reset()
                self.t.edit_modified(False)
            except:
                pass
        self.appTitle()


    def onSave(self):
        """ Der Menüpunkt 'Speichern' wurde aufgerufen """

        if self.t.edit_modified():
            if self.file is not None:
                try:
                    f = open(self.file, "w", encoding = self.encoding)
                    f.write(self.t.get('1.0', 'end'))
                    f.close()
                    self.t.edit_reset()
                    self.t.edit_modified(False)
                except:
                    messagebox.showwarning(self.appName, "Fehler beim Speichern der Datei !")

        elif self.file is not None:
            messagebox.showwarning(self.appName, "Der Text wurde nicht verändert !"+
                                        "\nDie Datei wurde nicht gespeichert.")

        else:
            self.file_saveas()

    def askSearch_callback(self, a):
#        print(a, type(a))
        if a is not None:
            self.searchStr   = a[0]
            self.replaceStr  = a[1]
            self.seekCase    = a[2]
            self.seekAll     = a[3]
            self.seekReplace = a[4]
            self.seekReg     = a[5]


    def onGoto(self, e = None):
        """ Sucht die angegebene Zeilennummer und selektiert diese """
        self.sel_start = None

        Input(self, self.setLine)

        if self.sel_start is not None:
            maxidx  = self.t.index(tk.END + '-1c')
            maxline = int(maxidx.split('.')[0])
            if self.sel_start[0] > 0 and self.sel_start[0] <= maxline:
                sel_start = "{}.{}".format(self.sel_start[0], self.sel_start[1])
                self.t.mark_set(tk.INSERT, sel_start)
                self.t.tag_remove(tk.SEL, '1.0', tk.END)

                # wenn nur der Anfang angegeben wurde
                if self.sel_end is None:
                    self.t.tag_add(tk.SEL, sel_start, sel_start + 'lineend')
                    self.t.see(tk.INSERT)

                # Wenn Anfang und Ende angegeben wurden
                else:

                    if self.sel_end[0] > 0 and self.sel_end[0] <= maxline:
                        sel_end = "{}.{}".format(self.sel_end[0], self.sel_end[1])
                        self.t.tag_add(tk.SEL, sel_start, sel_end)
                        self.t.see(sel_end)

            else:
                tk.messagebox.showerror(self.appName, "Ungültige Zeilennumer !")


    def setLine(self, line):
        """ Callback für den Dialog zur Auswahl der Zeilennummer """
        self.sel_start = line[0]
        self.sel_end   = line[1]


class Input(tk.Toplevel):
    def __init__(self, master, callback = None):
        tk.Toplevel.__init__(self)
        wpos = master.winfo_geometry().split('+')
        self.geometry("+{}+{}".format(int(wpos[1])+ 100, int(wpos[2]) + 100))

        self.resizable(width = None, height = None)
        self.protocol('WM_DELETE_WINDOW', self.quit)
        self.callback = callback

        x = tk.StringVar()
        tk.Label(self,width = 30, text = "Selektiere: Zeile / Position / Bereich", anchor = 'w').\
                grid(column = 1, row = 0, padx = 5, pady = 5, sticky ="nsw")
        self.entry = tk.Entry(self, textvariable = x, width = 12)
        self.entry.grid(column = 0, row = 0, padx = 5, pady = 5, sticky = "nsew")
        self.entry.bind('<Return>', self.onContinue)

        tk.Button(self, text = "Ausführen", command = self.onContinue, width = 12).\
                grid(column = 0, row = 1, padx = 5, pady = 5, sticky = "nse")
        tk.Button(self, text = "Abbrechen", command = self.quit, width = 12).\
                grid(column = 1, row = 1, padx = 5, pady = 5, sticky = "nsw")

        self.entry.focus_set()
        self.transient(master)
        self.grab_set()
        self.mainloop()
        self.destroy()


    def onContinue(self, e = None):
        search = self.entry.get()

        sel_start = None
        sel_end   = None
        error     = False

        search = search.split(',')

 #       print(search, len(search))

        if len(search) == 1:
            if search[0]:
                sel_start = search[0]

        elif len(search) == 2:
            sel_start = search[0]
            sel_end    = search[1]

        else:
            error = True

        if sel_start is not None:
            try:
                sel_start = sel_start.split('.')
                if len(sel_start) == 1:
                    sel_start = (int(sel_start[0]), 0)
                elif len(sel_start) == 2:
                    sel_start = (int(sel_start[0]), int(sel_start[1]))
                else:
                    error = True
            except:
                error = True

        if sel_end is not None:
            try:
                sel_end = sel_end.split('.')
                if len(sel_end) == 1:
                    sel_end = (int(sel_end[0]), 0)
                elif len(sel_end) == 2:
                    sel_end = (int(sel_end[0]), int(sel_end[1]))
                else:
                    error = True
            except:
                error = True

#        print(sel_start, sel_end)

        if not error:
            if self.callback is not None:
                self.callback((sel_start, sel_end))
            self.quit()

        else:
            tk.messagebox.showerror("Eingabefehler", \
                        "Ungültige Selektion : {} !\n"
                        "Format [[zeile] .spalte] [[,zeile] .spalte]".format(search))

class askSearch(tk.Tk):
    """
    Ein Fenster, in dem etwas Statistik, nämlich:
    - die Anzahl der durchsuchten Verzeichnisse
    - die Anzahl der durchsuchten Files etc.
    angezeigt wird.
    Das Fenster kann mit dem Button "Schließen" oder mit dem "X" beendet werden
    """
    init = ("", "", 0, 1, 0, 0)

    def __init__(self, master, callback = None, title = "", regex = True):
        tk.Toplevel.__init__(self)

        self.protocol('WM_DELETE_WINDOW', self.onCloseWindow)
        self.title(title + " - Suchen / Ersetzen")
        self.bind('<Escape>', self.onCloseWindow)
        self.callback = callback

        wpos = master.winfo_geometry().split('+')
        self.geometry("+{}+{}".format(int(wpos[1])+ 100, int(wpos[2]) + 100))
        self.resizable(width = None, height = None)

        self.f = tk.LabelFrame(self)
        self.f.grid(column = 0, row = 0)

        self.master = master
        self.search  = tk.StringVar()
        self.search.set(askSearch.init[0])
        self.replace = tk.StringVar()
        self.replace.set(askSearch.init[1])
        self.on_case = tk.IntVar()
        self.on_case.set(askSearch.init[2])
        self.on_seekAll= tk.IntVar()
        self.on_seekAll.set(askSearch.init[3])
        self.on_replace = tk.IntVar()
        self.on_replace .set(askSearch.init[4])
        self.on_seekReg = tk.IntVar()
        self.on_seekReg.set(askSearch.init[5])

        tk.Label(self.f, text = "Suchen nach :", width = 15, anchor = 'e' ). grid(
                column = 0, row = 1, padx = 5, pady = 5, sticky = "nse")
        tk.Label(self.f, text = "Ersetzen durch :" , width = 15, anchor = 'e').grid(
                column = 0, row = 2, padx = 5, pady = 5, sticky = "nsw")

        e1 = tk.Entry(self.f, textvariable = self.search, width = 30)
        e1. grid(column = 1, row = 1, padx = 5, pady = 5, sticky = 'nsew')
        e1.bind('<Return>', self.onContinue)
        e2 = tk.Entry(self.f, textvariable = self.replace, width = 30)
        e2.grid(column = 1, row = 2, padx = 5, pady = 5, sticky = 'nsew')
        e2.bind('<Return>', self.onContinue)

        tk.Checkbutton(self.f, variable = self.on_case,
                            text = "Groß-/Kleinschreibung ignorieren").\
                            grid(column = 2, row = 0, padx = 5, pady = 5, sticky = "nsw")
        tk.Checkbutton(self.f, variable = self.on_seekAll,
                            text = "Alle Fundstellen auswählen").\
                            grid(column = 2, row = 1, padx = 5, pady = 5, sticky = "nsw")

        tk.Checkbutton(self.f, variable = self.on_replace,
                            text = "Fundstellen ersetzen").\
                            grid(column = 2, row = 2, padx = 5, pady = 5, sticky = "nsw")

        if regex: regex = tk.NORMAL
        else:     regex = tk.DISABLED

        tk.Checkbutton(self.f, variable = self.on_seekReg, state = regex,
                            text = "Regular Expressions auswerten").\
                            grid(column = 2, row = 3, padx = 5, pady = 5, sticky = "nsw")

        tk.Button(self.f, text = "Ausführen", command = self.onContinue, width = 13).\
                grid(column = 1, row = 3, padx = 5, pady = 5, sticky = "nsw")
        tk.Button(self.f, text = "Abbrechen", command = self.onCloseWindow, width = 13).\
                grid(column = 1, row = 3, padx = 5, pady = 5, sticky = "nse")

        self.transient(self.master)
        self.grab_set()
        e1.focus_set()

        self.mainloop()
        self.destroy()


    def onContinue(self, e = None):
        if self.callback is not None:
            init = (self.search.get(), self.replace.get(), self.on_case.get(),
                self.on_seekAll.get(), self.on_replace.get(), self.on_seekReg.get())
            # die aktuellen Einstellung sichern
            askSearch.init = init
            self.callback(init)
        self.quit()


    def onCloseWindow(self, e = None):
        """ Gibt None zurück, die Einstellungen werden nicht gesichert """
        if self.callback is not None:
            self.callback(None)
        self.quit()


if __name__ == "__main__":
    from io import StringIO

    t = tk.Tk()

    # - Den Editor mit einem String öffnen
    #Editor(t, text = "Dies ist der Testtext,\nder auch eine zweite Zeile hat\nsdfhkj\ndfkjh\nsdf  !")

    # - Den Editor mit einer Datei öffnen
    Editor(t, file = "test2.txt")

    # - Den Editor mit einem StringIO öffnen
##    f = StringIO()
##    f.write("Dies ist der Testtext,\nder auch eine zweite Zeile hat\nsdfhkj\ndfkjh\nsdf  !" )
##    Editor(t, file = f, protect = True)
##    f.close()

    t.destroy()

