#! /usr/bin/python3
# -*- coding: UTF-8 -*-
# 21.09.2014

import tkinter as tk
from tkinter import filedialog

import os, time, datetime, io
from editor import Editor, askSearch
import my_dialogs
import shutil as sh
import scripts

def Script_copy(master):
    """
    Kopiert alle Dateien, die in 'tmp.tmp' aufgelistet sind.
    Das StartDir für diese Dateien steht in der 1.Zeile vom tmp.tmp,
    es ist nicht der Eintrag self.startDir im Root-Window !
    Das BackupDir wird dagegen aus dem Root-Window geholt.
    """

    # Holt die Files als Liste und das Startverzeichnis der Dateien
    files, startDir = scripts.extract_files(master)

    pathvar = 0
    copyvar = 0
    simvar  = 1
    # Die Einstellung des BackupDirs holen
    master.check_backDir()
    backDir = master.backDir

    def callback(a):
        """
        Holt die Eingaben aus dem CopyFileWindow ab
        """
        nonlocal pathvar, copyvar, simvar, backDir
        pathvar = a[0]
        copyvar = a[1]
        simvar  = a[2]
        backDir = a[3]

    # Dialog zur Auswahl der Parameter
    CopyFileWindow(master, backDir, startDir, callback)

    # Wenn die Eingabe abgebrochen wurde, dann wird (-1,-1,"") zurückgegeben
    if pathvar < 0:
        return

    if simvar:
        f = io.StringIO()
    else:
        f = None

    cnt_copy    = 0
    cnt_omit    = 0
    cnt_rename  = 0
    cnt_replace = 0

    for file in files:

        # der einfache Dateiname incl. path ab Startdir - > file
        _file = os.path.basename(file)      # der Dateiname
        _startpath = os.path.splitdrive(startDir)[1]
        if _startpath.startswith('\\'):
            _startpath = _startpath[1:]

#        print(backDir, _startpath, _file, file)

        # Dies ist immer die zu kopierende Datei mit absolutem Pfad
        source = os.path.join(startDir, file)
 #       print(source)

        if pathvar == 0:                    # relative Pfade verwenden
            target = os.path.join(backDir, file)

        elif pathvar == 1:                  # Absolute Pfade verwenden
            target = os.path.join(backDir, _startpath, file)

        elif pathvar == 2:                  # ohne Pfad kopieren
            target = os.path.join(backDir, _file)

        # was tun, wenn die Zieldatei bereits existiert ...
        if os.path.isfile(target):
            if copyvar == 0:                # Index anhängen
                target = scripts.make_fileindex(target)
                if simvar:
                    f.write("umbenannt   : {}\n".format(target))
                cnt_rename += 1

            elif copyvar == 1:              # überschreiben
                if simvar:
                    f.write('ersetzt     : {}\n'.format(target))
                cnt_replace += 1

            elif copyvar == 2:              # überspringen
                s = "ausgelassen : {}\n".format(target)
                if simvar:
                    f.write(s)
                # keine Dateien überschreiben !
                cnt_omit += 1
                continue
        else:
            p = os.path.split(target)[0]
            # falls das Verzeichnis nicht existiert
            if not os.path.exists(p):
                if simvar:
                    f.write('makeDirs    : {}\n'.format(p))
                else:
                    os.makedirs(p)

            if simvar:
                f.write('kopiert     : {}\n'.format(target))
            cnt_copy += 1

        if not simvar:
            if target is not None:
                sh.copy2(source, target)
            else:
##                tk.messagebox.showwarning(master.appName, "Dateiname = None")
                return

    if simvar:
        f.write("\n\nkopiert     : {:4}\n".format(cnt_copy))
        f.write("umbenannt   : {:4}\n".format(cnt_rename))
        f.write("ersetzt     : {:4}\n".format(cnt_replace))
        f.write("ausgelassen : {:4}\n".format(cnt_omit))

        Editor(master, file = f)
        f.close()

    else:
        tk.messagebox.showinfo(master.appName,
            ( "Es wurden insgesamt\n" +
              "{} Dateien kopiert,\n" +
              "{} Dateien umbenannt,\n"+
              "{} Dateien ersetzt,\n" +
              "{} Dateien ausgelassen !").\
            format(cnt_copy, cnt_rename, cnt_replace, cnt_omit))


class CopyFileWindow(tk.Tk):
    init = (0,0)
    def __init__(self, master, backDir, startDir, callback):
        tk.Toplevel.__init__(self)
        self.protocol('WM_DELETE_WINDOW', self._quit)
        self.title(master.appName + ": Dateien kopieren")

        wpos = master.winfo_geometry().split('+')
        self.geometry("+{}+{}".format(int(wpos[1])+ 100, int(wpos[2]) + 150))
        self.resizable(width = None, height = None)
        self.bind('<Escape>', self._quit)

        self.backDir = tk.StringVar()
        self.backDir.set(backDir)
        self.callback = callback

        # Pfade
        self.pathvar = tk.IntVar()
        self.f1 = tk.LabelFrame(self, text = "Dateipfade")
        self.f1.grid(column = 0, columnspan = 3, row = 0, padx = 5,pady = 5, sticky = "nsew")

        tk.Radiobutton(self.f1, variable = self.pathvar, value = 0, anchor = 'w',
            text = "Dateien mit relativem Pfad zu {} kopieren".format(startDir)).\
            grid(column = 1, columnspan = 2, row = 1, padx = 5, pady = 5, sticky = "nsew")

        tk.Radiobutton(self.f1, variable = self.pathvar, value = 1 , anchor = 'w',
            text = "Dateien mit absolutem Pfad ins Zielverzeichnis kopieren").\
            grid(column = 1, columnspan = 2, row = 2, padx = 5, pady = 5, sticky ="nsew")

        tk.Radiobutton(self.f1, variable = self.pathvar, value = 2 , anchor = 'w',
            text = "Dateien ohne Pfad direkt ins Zielverzeichnis kopieren").\
            grid(column = 1, columnspan = 2, row = 3, padx = 5, pady = 5, sticky ="nsew")

        # Kopieren
        self.copyvar = tk.IntVar()
        self.f2 = tk.LabelFrame(self, text = "Wenn der Dateiname bereits existiert, dann ... ")
        self.f2.grid(column = 0, columnspan = 3, row = 1, padx = 5,pady = 5, sticky = "nsew")

        tk.Radiobutton(self.f2, variable = self.copyvar, value = 0, anchor = 'w',
            text = "Bestehende Datei beibehalten, kopierte Datei automatisch mit Index erweitern").\
            grid(column = 1, columnspan = 2, row = 1, padx = 5, pady = 5, sticky = "nsew")

        tk.Radiobutton(self.f2, variable = self.copyvar, value = 1 , anchor = 'w',
            text = "Bestehende Datei durch die neue Datei überschreiben").\
            grid(column = 1, columnspan = 2, row = 2, padx = 5, pady = 5, sticky ="nsew")

        tk.Radiobutton(self.f2, variable = self.copyvar, value = 2 , anchor = 'w',
            text = "Bestehende Datei beibehalten, neue Datei nicht kopieren").\
            grid(column = 1, columnspan = 2, row = 3, padx = 5, pady = 5, sticky ="nsew")

        tk.Label(self, text = "Zielverzeichnis:", anchor = 'e', width = 13).\
             grid(column = 0, row = 2, padx = 5, pady = 5, sticky = "nsew")

        tk.Label(self, textvariable = self.backDir, width = 45, anchor = 'w' ).\
              grid(column = 1, row = 2, padx = 5, pady = 5, sticky = "nsw")

        btn1 = tk.Button(self, text = "...", command = self.onDir, width = 12)
        btn1.grid(column = 2, row = 2, padx = 5, pady = 5, sticky = "nsw")

        self.chkvar = tk.IntVar()
        self.chkvar.set(1)
        chk = tk.Checkbutton(self, variable = self.chkvar,
                            text = "Das Kopieren nur simulieren", anchor = 'w')
        chk.grid(column = 0, columnspan = 2, row = 4, padx = 5, pady = 5, sticky = 'nsew')

        btn2 = tk.Button(self, text = "Kopieren", command = self.onCopy, width = 12)
        btn2.grid(column = 1, row = 4, padx = 5, pady = 5, sticky = "nse")
        btn2.bind('<Return>', self.onCopy)
        btn2.focus_set()

        btn3 = tk.Button(self, text = "Abbrechen", command = self._quit, width = 12)
        btn3.grid(column = 2, row = 4, padx = 5, pady = 5, sticky = "nsw")
        btn3.bind('<Return>', self._quit)

        self.pathvar.set(CopyFileWindow.init[0])
        self.copyvar.set(CopyFileWindow.init[1])

        self.transient(master)
        self.grab_set()
        self.mainloop()
        self.destroy()


    def onCopy(self, e = None):
        CopyFileWindow.init = (self.pathvar.get(), self.copyvar.get())
        self.callback( (self.pathvar.get(),  self.copyvar.get(), \
                    self.chkvar.get(), self.backDir.get()) )
        self.quit()


    def onDir(self):
        dir = filedialog.askdirectory(title = \
            "Das Zielverzeichnis für die Kopien auswählen :")
        if dir:
            dir = os.path.normcase(dir)
            self.backDir.set(dir)


    def _quit(self, e = None):
        self.callback((-1, -1, 1, ""))
        self.quit()


