#! /usr/bin/python3
# -*- coding: UTF-8 -*-
# 21.09.2014

import tkinter as tk
from tkinter import filedialog
import os, time, datetime, io, threading, queue
import zipfile as z
import scripts, simplebargraph
# Der Dateiname wird aus dem Namen des Startverzeichnis + Datum gebildet
# Wenn Root ausgewählt als Startverzeichnis, dann wird als Name gewählt:
DEFAULT_ZIP_NAME = "back"

def splitLastPath(path):
    """
    Der Vorgabename für ein Zipfile ist oberste Ebene des Namens des Startverzeichnis.
    Ist der Name "", dann wird der DEFAULT_ZIP_NAME zurückgegeben.
    """
    path = path[path.rfind('\\') + 1:]
    if not path:
        path = DEFAULT_ZIP_NAME
    return path


class ZipFiles(object):
    def __init__(self, master):
        """
        Schreibt Dateien in ein ZipFile.
        Die Dateien werden in der Datei master.tmpFile übergeben.
        Ein Dateiname wird aus dem DEFAULT_ZIP_NAME + Datum [+ Index]
        automatisch gebildet.
        Als Zielverzeichnis wird das backDir vorgeschlagen,
        Verzeichnis und Dateinamen kann aber nach Bedarf geändert werden.
        """
        self.master = master
        self.cwd = os.getcwd()      # Damit das cwd am Ende restauriert werden kann
        self.pathvar = 0
        self.zipfile = 0
        self.wait = tk.StringVar()

        # Holt die Files als Liste und das startDir der Files ab
        files, startDir = scripts.extract_files(master)

        if files:
            # Einen Vorschlag für einen eindeutigen ZipFileNamen erstellen
            # Der besteht aus einem String mit angehängtem Datum + 'zip'
            # -> back140913.zip

            master.check_backDir()
            self.backDir = master.backDir

            d = str(datetime.date.today())
            name = "{}_{}{}{}".format(splitLastPath(startDir), d[2:4], d[5:7], d[8:])
            name = os.path.join(self.backDir, name)
            self.zipfile = "{}.zip".format(name)

            # falls der Dateiname schon besteht, dann einen Index anhängen
            if os.path.isfile(self.zipfile):
                self.zipfile = scripts.make_fileindex(self.zipfile)

            ArchivFileWindow(master, self.zipfile, startDir, self.callback)

            if not self.zipfile:
                return

            self.bg = simplebargraph.SimpleBarGraphWindow(master, \
                                            callback = self.viewerStop)
            self.bg.showStatus()

            if not self.pathvar:                 # relativer Pfad
                _cwd = os.getcwd()
                os.chdir(startDir)

            self.queue = queue.Queue()

            self.zip = ZipWriter(self.zipfile, files, self.pathvar, startDir,
                                            self.wait, self.queue)
            self.zip.daemon = True
            self.zip.start()

            self.master.after(50, self.readQueue)

            # ----- warten bis zum Ende des Threads ZipWriter
            self.master.wait_variable(self.wait)

            self.bg.killWindow()
            os.chdir(self.cwd)

            try:
                x = int(self.wait.get())
                tk.messagebox.showwarning(master.appName + " Zip-Script", \
                        '{} Dateien wurden gesichert im Archiv\n\n{}'\
                        .format(len(files), self.zipfile))
            except:
                tk.messagebox.showwarning(master.appName + ": Zip-Script", \
                        (('Fehler: {} !\n\n' +\
                        'Datei: {}\n\n' +\
                        'Die Sicherung ist nicht vollständig !').\
                        format(self.wait.get(), self.zipfile)))

        else:
            tk.messagebox.showwarning(master.appName + ": Zip-Script", \
                "Die Dateiliste war leer,\nes wurden keine Dateien gesichert !")


    def readQueue(self):
        """
        Gibt die Daten aus der Queue weiter an das anzeigende ToplevelWindow
        Allerdings wird diese Aktualisierung nur in größeren Zeitabständen
        ausgeführt, alle zwischendrin eingegangenen Einträge werden
        ausgelesen, aber ignoriert.
        """

        input = ""
        self.queue_tmp = ""
        queue_continue = True
        try:
            # liest solange die Queue aus, bis keine Daten mehr anstehen,
            # der letzte Datensatz steht dann in self.queue_tmp
            # und wird von except ausgewertet.
            while True:
                input = self.queue.get(block = False)
                # None ist das Zeichen für den Abschluss des Threads
                if input is None:
                    self.bg.showStatus(self.queue_tmp)
                    queue_continue = False
                # der letzte Datensatz wird zwischengespeichert
                else:
                    self.queue_tmp = input

        except queue.Empty:
            # None ist das Zeichen für den Abschluss des Threads
            if input is not None:
                self.bg.showStatus(input)

            else:
                self.bg.showStatus(self.queue_tmp)
                queue_continue = False

        if queue_continue:
            self.master.after(150, self.readQueue)


    def callback(self, a):
        """
        Holt die Einstellungen aus dem ArchivFileWindow ab
        """
        self.pathvar = a[0]
        self.zipfile = a[1]


    def viewerStop(self):
        """ Wird aufgerufen, wenn der Progressbar manuell beendet wird """
        self.zip.killThread()
        self.bg.killWindow()


class ZipWriter(threading.Thread):
    """
    Ein Thread, der im Hintergrund die zu archivierenden Dateien
    in ein Zip-File schreibt.
    Dabei wird nach jeweils 10 Dateien ein Eintrag in der Queue abgelegt,
    der vom aufrufenden Programm ausgewertet wird zur Aktualisierung des
    Progressbars führt.
    """
    def __init__(self, zipfile, files, pathvar, startDir, wait, queue):
        threading.Thread.__init__(self)
        self.zipfile  = zipfile
        self.files    = files
        self.pathvar  = pathvar
        self.startDir = startDir
        self.wait     = wait
        self.queue    = queue
        self.go       = True


    def run(self):
        try:
            zip = z.ZipFile(self.zipfile, 'w', compression=z.ZIP_DEFLATED)
            cnt = 0
            for file in self.files:
                if self.pathvar:
                    zip.write(os.path.join(self.startDir, file))

                else:
                    zip.write(file)

                cnt += 1
                if not cnt % 10:        # nach jeweils 10 Dateien ein Update
                    self.queue.put(1)   # der Wert ist belanglos
                                        # solange nicht 'None' gesendet wird
                if not self.go:
                    raise StopIteration

        except FileNotFoundError:
            ret = '{} nicht gefunden'.format(file)
        except StopIteration:
            ret = "User-Abbruch"
        except:
            ret = "ZipWrite Error"
        else:
            ret = len(self.files)
        finally:
            # Das Ende signalisieren
            self.queue.put(None)
            zip.close()
            # self.wait gibt Auskunft über den Erfolg des Threads
            # das aufrufende Programm wartet mit wait_variable() auf
            # die Änderung der Variablen und wertet sie aus
            self.wait.set(ret)


    def killThread(self):
        self.go = False


class ArchivFileWindow(tk.Tk):
    init = 0        # Vorgabe beim Programmstart: Relative Pfade zu StartDir

    def __init__(self, master, zipfile, startDir, callback):
        tk.Toplevel.__init__(self)
        self.protocol('WM_DELETE_WINDOW', self._quit)
        self.title(master.appName + ": Dateien komprimieren")
        self.master = master

        wpos = master.winfo_geometry().split('+')
        self.geometry("+{}+{}".format(int(wpos[1])+ 100, int(wpos[2]) + 150))
        self.resizable(width = None, height = None)
        self.bind('<Escape>', self._quit)

        self.zipfile = tk.StringVar()
        self.zipfile.set(zipfile)
        self.callback = callback


        # Pfade
        self.pathvar = tk.IntVar()
        self.f1 = tk.LabelFrame(self, text = "Dateipfade")
        self.f1.grid(column = 0, columnspan = 3, row = 0, padx = 5,pady = 5, sticky = "nsew")

        tk.Radiobutton(self.f1, variable = self.pathvar, value = 0, anchor = 'w',
            text = "Dateien relativ zu Pfad {} archivieren".format(startDir)).\
            grid(column = 1, columnspan = 2, row = 1, padx = 5, pady = 5, sticky = "nsew")

        tk.Radiobutton(self.f1, variable = self.pathvar, value = 1 , anchor = 'w',
            text = "Dateien mit absolutem Pfad archivieren").\
            grid(column = 1, columnspan = 2, row = 2, padx = 5, pady = 5, sticky ="nsew")


        # Die erste Zeile
        tk.Label(self, text = "BackupDatei:", anchor = 'e', width = 13).\
             grid(column = 0, row = 2, padx = 5, pady = 5, sticky = "nsew")

        tk.Label(self, textvariable = self.zipfile, width = 45, anchor = 'w' ).\
              grid(column = 1, row = 2, padx = 5, pady = 5, sticky = "nsw")

        btn1 = tk.Button(self, text = "...", command = self.onDir, width = 12)
        btn1.grid(column = 2, row = 2, padx = 5, pady = 5, sticky = "nsw")


        btn2 = tk.Button(self, text = "Archivieren", command = self.onArchiv, width = 12)
        btn2.grid(column = 1, row = 4, padx = 5, pady = 5, sticky = "nse")
        btn2.bind('<Return>', self.onArchiv)
        btn2.focus_set()

        btn3 = tk.Button(self, text = "Abbrechen", command = self._quit, width = 12)
        btn3.grid(column = 2, row = 4, padx = 5, pady = 5, sticky = "nsw")
        btn3.bind('<Return>', self._quit)

        self.pathvar.set(ArchivFileWindow.init)

        self.transient(master)
        self.grab_set()
        self.mainloop()
        self.destroy()


    def onArchiv(self, e = None):
        """ Button 'Ausführen' gedrückt """
        ArchivFileWindow.init = (self.pathvar.get())
        self.callback( (self.pathvar.get(), self.zipfile.get()) )
        self.quit()


    def _quit(self, e = None):
        """ Button 'Abbrechen', ESC oder 'X' gedrückt """
        # Die Rückgabe von zipfile = "" führt zum Beenden des Scripts
        self.callback((-1, ""))
        self.quit()


    def onDir(self):
        """ Button '...' gedrückt """
        dir = filedialog.asksaveasfilename(defaultextension = '.zip',\
                    title = self.master.appName + ": Zip-Datei schreiben",\
                    initialfile = self.zipfile.get(),\
                    filetypes =[("ZIP", "*.zip")] )
        if dir:
            self.zipfile.set(dir)
