#! /usr/bin/python3
# -*- coding: UTF-8 -*-
# 21.09.2014

import tkinter as tk
import os
from editor      import Editor
from script_zip  import ZipFiles
from script_copy import Script_copy
from script_seek import SeekByStringFunctions, SeekByTextObject

binfiles = ('exe', 'com', 'zip', 'dll', 'drv', 'chm' )
txtfiles = ('py', 'txt', 'c', 'h', 'ini', 'html', 'htm' )

def check_ext(master, file):
    """
    Prüft, ob die übergebene Dateierweiterung registiert ist:
    - in der Positivliste als lesbare Textdatei
    - in der Negativliste als Binärdatei
    Rückgabe:
    - True wenn Textdatei oder unbekannte Datei, die zum Öffnen bestätigt wurde
    - False wenn Binärdatei oder abgelehnte unbekannte Datei
    """
    ext = os.path.splitext(file)[1][1:]

    if ext in txtfiles:         # eine lesbare Datei
        return True

    if ext in binfiles:         # eine Binärdatei
        tk.messagebox.showwarning(master.appName,
               "Die Datei {}\n".format(file) +\
               "ist eine Binärdatei und kann nicht geöffnet werden !")
        return False

    else:                       # eine nicht registierte Datei
        return tk.messagebox.askyesno(master.appName,
                    "Die Datei {}\n".format(file) +\
                         "ist möglicherweise keine Textdatei.\nTrotzdem öffnen ?")


def extract_files(master):
    """
    Öffnet die Temp-Datei,
    liest aus der ersten Zeile das Startverzeichnis
    und anschließend alle Dateinamen in ein Array
    Rückgabe:
    - Array mit Pfad ab Startdir und Dateinamen
    - Das Startverzeichnis, das mit join() vor den Dateinamen gesetzt werden muss
    """
    file = master.tmpFile
    if not os.path.isfile(file):
        tk.messagebox.showwarning(master.appName + ": FileCopy",
            "Es ist kein Temp-File vorhanden !")
        return

    try:
        f = open(file, 'r', encoding = 'utf-8')
        line = f.readline()
        startDir = line[1: line.rfind('+')].strip()
        files = []

        while line:
            line = f.readline()
            if not line.startswith('#'):
                files.append(line[50:].strip())
            else:
                break
    finally:
        f.close()

    return files, startDir


def make_fileindex(file):
    """
    Die Datei 'file' besteht bereits,
    der Name soll durch anhängen eines Index im Format '_x'
    geändert so werden, dass der Dateiname wieder eindeutig ist
    und die bestehende Datei nicht überschrieben wird.

    Todo: der Index ist nur im Bereich [a-z] vorgesehen

    Rückgabe: der neue Dateiname - oder None
    """
    name, ext = os.path.splitext(file)
    i = ord('a')
    while i <= ord('z'):
        newfile = '{}_{}{}'.format(name, chr(i), ext)
        i += 1

        if not os.path.isfile(newfile):
            break
        else:
            newfile = None

    if newfile is None:
        tk.messagebox.showwarning("tools/make_fileindex()", \
                    "Index größer als 'z' -> 'None'")

    return newfile


def viewFiles(master):
    """
    ruft die in master.tmpFile aufgeführten Dateien nacheinander im Editor auf
    """

    files, startDir = extract_files(master)
    filecnt = len(files)
    cnt = 0
    for file in files:
        cnt += 1
        file = os.path.join(startDir, file)

        reply = tk.messagebox.askyesnocancel(master.appName,
        "Datei # {} von {} Dateien.\n\n{}\n\nim Editor öffnen ?\n\n".\
            format(cnt, filecnt, file))

        if reply is None:
            return

        elif reply:
            # Prüft die Dateierweiterung, nur gültige Dateien werden geöffnet
            if check_ext(master, file):
                Editor(master, file = file)


def viewTmpFile(master):
    """ ruft das master.tmpFile im Editor auf """
    Editor(master, file = master.tmpFile)


class Script_default(object):
    def __init__(self, master):
        """
        Der master stellt alle relevanten Informationen zur Verfügung
        # master =
        # master.startDir
        # master.backDir
        """
        self.master = master

##        self.start()
##
##    def start(self):
        print("scripts.py -> 'Script_default'")
        print(self.master)
        print(self.master.startDir)
        print(self.master.check_backDir())
        tk.messagebox.showinfo(self.master.appName, \
                        "Dies ist nur eine Vorlage ohne Funktion")


# ---------------------------------------
# Die Definition der Scripte:
# ein Bezeichner für die Combobox und die zugehörige Funktion
# wird aus walker_gui importiert
scripts_dict = { 'Dateien zippen'    : ZipFiles,
                 'Dateien öffnen'    : viewFiles,
                 'StringSuche [1]'   : SeekByStringFunctions,
                 'StringSuche [2]'   : SeekByTextObject,
                 'TempDatei öffnen'  : viewTmpFile,
                 'default_Script'    : Script_default,
                 'Dateien kopieren'  : Script_copy,}

