#!/usr/bin/python
# -*- coding: UTF-8 -*-
# 21.09.2014

import tkinter as tk

class SimpleBarGraphWindow(tk.Toplevel):
    """
    Ein einfacher Bargraph, der nur eine Ausrichtung kennt und fortlaufend
    incrementiert, am Ende auf 0 überläuft und von vorne beginnt.
    Methoden:
        - init(master, callback)
        - showStatus()
        - closeWindow()
        - killWindow()
    """
    def __init__(self, master,
                 callback = None,
                 title = "Bargraph",
                 width = "300",             # Ausdehnung in X-Richtung in Pixeln
                 heigth = "20",             # Ausdehnung in y-Richtung in Pixeln
                 bg_color  = "#D8D8D8",     # die Hintergrundfarbe
                 bar_color = "#B0B0C0",):   # die Farbe des Balkens

        tk.Toplevel.__init__(self)
        self.protocol('WM_DELETE_WINDOW', self.closeWindow)

        wpos = master.winfo_geometry().split('+')
        self.geometry("+{}+{}".format(int(wpos[1])+ 100, int(wpos[2]) + 100))
        self.title(title)
        self.resizable(width = None, height = None)

        self.callback = callback
        self.value = 0

        self.__start_value = 0
        self.__end_value   = 100
        self.__pixel_x     = width
        self.__pixel_y     = heigth
        self.__length = int(self.__pixel_x) # die max. Länge des Bargraphen

        self.__bg_color    = bg_color
        self.__bar_color   = bar_color

        self.frm = tk.LabelFrame(self, text = "Fortschritt ... ", height = 50)
        self.frm.grid(column = 0, row = 0, padx = 5, pady = 5)

        self.canvas = tk.Canvas(self.frm, width = width, height = heigth)
        self.canvas.grid(column = 0, row = 0, padx = 5, pady = 5, sticky = "w")

        # die Hintergrundfläche ist unabhängig von der Richtung des Bargraphen
        #self.__bargnd =
        self.canvas.create_rectangle("0","0", width, heigth, fill = bg_color)

        # der Bargraph wird als leere Fläche eingerichtet, die mit showStatus()
        # gezeichnet werden mus
        self.__bar= self.canvas.create_rectangle("0", "0", "0",  "0", fill = bar_color)

        tk.Button(self.frm, text = "abbrechen", command = self.closeWindow, \
                width = 15, anchor = tk.CENTER).\
                grid(column = 0, row = 1, padx = 5, pady = 5, sticky = "nse")

        self.transient(self.master)
        self.focus_set()
        self.grab_set()
        #self.wait_window()

    def ende(self):
        """ Signalisiert dem Master, dass er die Aktion beenden soll """
        if self.callback is not None:
            self.callback()

    def showStatus(self, status = 0):
        """
        Incrementiert den Fortschrittsbalken
        Der übergebene Parameter 'status' wird nicht ausgewertet.
        """
        # Falls das Fenster bereits geschlossen wurde bevor die Queue
        # beendet wurde, wird hier eine Fehlermeldung unterdrückt.
        try:
            self.value += 5
            bar_x = self.__end_value - self.__start_value
            bar_x = (self.value - self.__start_value) * self.__length // bar_x

            self.canvas.coords(self.__bar, "0", "0", str(bar_x), self.__pixel_y)

            if self.value > 100:
                self.value = -5
            self.update_idletasks()
        except:

            pass

    def closeWindow(self):
        """ Bereitet das Löschen des Toplevel-Windows vor
            wenn ein Callback existiert, dann wird dies ausgeführt
            und ist für das endgültige Schließen des Fensters zuständig
            (vorher müssen ggf. noch andere Aktionen ausgeführt werden).
        """
        if self.callback is not None:
            self.callback()
        else:
            self.destroy()


    def killWindow(self):
        """ endgültiges Schließen des Fensters durch den Master """
        self.destroy()


if __name__ == '__main__':

    i = 0
    def update():
        global after, i
        i += 1
        s.showStatus((0, i))
        after = root.after(500, update)
        print('root: updated')

    def kill():
        global after
        root.after_cancel(after)
        root.after_cancel(afterall)

        s.killWindow()
        print('root: destroyed')
        root.destroy()

    root = tk.Tk()

    s = SimpleBarGraphWindow(master = root, callback = kill)

    after = root.after(500, update)
    afterall =root.after(5300, kill)

    root.mainloop()