#! /usr/bin/python3
# -*- coding: UTF-8 -*-
# 21.09.2014

import datetime, os
import tkinter as tk
from tkinter import ttk

# Bei der Eingabe des Dateialters kann ein m/M, h/H, s/S, d/D, t/T, y/Y oder j/J
# angehängt werden, dann wird auf Sekunden hochgerechnet.
# Ohne Angabe wird von Stunden ausgegangen.
tab1 = {"M": 60,
        "H": 3600,
        "S": 3600,
        "D": 3600 * 24,
        "T": 3600 * 24,
        "Y": 3600 * 24 * 365,
        "J": 3600 * 24 * 365 }

def check_time_entry(time):
    """
    Prüft den Zeiteintrag, ob eine nachfolgende Angabe zur Zeiteinheit vorhanden ist,
    das kann Minute, Hour, Stunde, Day, Tag, Year oder Jahr sein.
    Ohne Angabe werden Stunden angenommen.
    Bei einer ungültigen Eingabe wird None zurückgegeben
    """
    t = time.strip()
    # eine leere Eingabe wird zu 0
    if not t:
        return 0

    # das letzte Zeichen im Dictionary suchen, wenn es nicht vorhanden ist,
    # dann wird 0 zurückgegeben
    faktor =  tab1.get(t[-1:].upper(), 0)
    if faktor:
        try:
            # das letzte Zeichen abschneiden und daraus einen Integer machen
            return (int(time[:-1])) * faktor
        except:
            pass
    else:
        try:
            return int(time) * 3600
        except:
            pass
    return None


# Bei der Einngabe der Dateigröße kann ein b/B, k/K, oder m/M angehängt werden
# dann wird die Eingabe auf Bytes umgerechnet.
# Ohne Angabe wird von Bytes ausgegangen.
tab2 = {"B" : 1,
        "K" : 1024,
        "M" : 1024 * 1024}


def check_size_entry(size):
    """ Hier wird die Eingabe der Dateigröße darauf überprüft,
        ob ein angehängtes Zeichen die Einheit der Eingabe als Kilo oder Megabyte
        interprtieren soll.
        Vorgabe ist die Angabe der Größe in Byte.
        Bei einer ungültigen Eingabe wird None zurückgegeben.
        """
    s = size.strip()
    if not s:
        return 0

    faktor = tab2.get(s[-1:].upper(), 0)
    if faktor:
        try:
            return (int(size[:-1])) * faktor
        except:
            pass
    else:
        try:
            return int(size)
        except:
            pass
    return None


class FileCounter(tk.Tk):
    """
    Ein Fenster, in dem etwas Statistik, nämlich:
    - die Anzahl der durchsuchten Verzeichnisse
    - die Anzahl der durchsuchten Files etc.
    angezeigt wird.
    Das Fenster kann mit dem Button "Beendet" oder mit dem "X" beendet werden
    """
    def __init__(self, master, callback = None, title = "..."):
        tk.Toplevel.__init__(self, master)

        self.protocol('WM_DELETE_WINDOW', self.closeWindow)
        self.title(title)
        self.callback = callback

        wpos = master.winfo_geometry().split('+')
        self.geometry("+{}+{}".format(int(wpos[1])+ 100, int(wpos[2]) + 100))
        self.resizable(width = None, height = None)

        self.f = tk.LabelFrame(self, width = 600, height = 400)
        self.f.grid(column = 0, row = 0)

        # Die erste Zeile ist konstant
        tk.Label(self.f, text = "StartDir :", width = 30, anchor = 'e' ). grid(
            column = 0, row = 0, padx = 5, pady = 5, sticky = "nse")
        tk.Label(self.f, text = master.startDir, width = 40, anchor = 'w').grid(
            column = 1, row = 0, padx = 5, pady = 5, sticky = "nsw")

        items = (("Durchsuche Verzeichnis :",),
                 ("Anzahl durchsuchter Verzeichnisse :",),
                 ("Anzahl gefilterter Verzeichnisse :",),
                 ("Anzahl der leeren Verzeichnisse:",),
                 ("Anzahl gefundener Dateien :",),
                 ("Anzahl gefilterter Dateien :",),)
        self.labels = []
        row = 1
        for item in items:
            self.labels.append(tk.StringVar())
            tk.Label(self.f, text = item[0], width = 30, anchor = 'e' ). grid(
                column = 0, row = row, padx = 5, pady = 5, sticky = "nse")
            tk.Label(self.f, textvariable = self.labels[row - 1], width = 40, anchor = 'w').grid(
                column = 1, row = row, padx = 5, pady = 5, sticky = "nsw")
            row += 1

        tk.Button(self.f, text = "Beenden", command = self.closeWindow, width = 20).\
                grid(column = 1, row = row, padx = 5, pady = 5, sticky = "nsw")

        self.transient(master)
        self.grab_set()
        self.focus_set()


    def showStatus(self, args):
        """
        Hier wird die Anzeige aktualisiert, es wird ein Tuple mit den letzten
        Werten übergeben.
        """
        try:
            for i in range(6):
                self.labels[i].set(args[i])
            self.update_idletasks()

        except:
            if len(args):
                print("FileCounter: Error in args: {}".format(args))


    def closeWindow(self):
        """
        Ein Click auf den Button soll das (ggf. vorzeitige) Ende einläuten.
        Dazu wird eine Nachricht zum Master gesenden, der kann dann den Thread
        beenden oder sonstige Schlüsse ziehen.
        Anschließend wird das eigene Fenster geschlossen.
        """
        if self.callback is not None:
            self.callback()
        else:
            self.destroy()

    def killWindow(self):
        self.destroy()



if __name__ == "__main__":
    # Alle Parameter sind jeweils vom Typ String
    print(check_size_entry('1m'))
