#! /usr/bin/python3
# -*- coding: UTF-8 -*-
# 21.09.2014

import tkinter as tk
from tkinter import ttk

import os, pickle, time, threading, queue
from io import StringIO

from editor import *
from walker import FileVisitor
from simplebargraph import SimpleBarGraphWindow
from tools import *
from editor import askSearch
import scripts

# in dieser Datei können die Programm-Einstellungen gespeichert werden,
# sie werden dann bei einem Programmstart geladen
DEFAULT_INI_FILE = "default.wlk"
# In dieser Datei werden die Suchergebnisse gespeichert
# Diese Datei wird von den Scripten geöffnet
DEFAULT_TMP_FILE = "tmp.tmp"
# Dies ist der Fenstertitel
APP_NAME = "FileWalker"

class gui(tk.Tk):
    """
    Grafische Oberfläche zur Eingabe und Prüfung von Parametern, die anschließend
    an "walker.py" zur Ausführung / Dateiauswahl übergeben werden.
    """
    def __init__(self):
        tk.Tk.__init__(self)
        self.appName = APP_NAME
        self.title(self.appName)
        self.resizable(width = False, height = False)
        self.protocol('WM_DELETE_WINDOW', self.onQuit)
        self.bind('<Escape>', self.onQuit)

        self.frm = tk.LabelFrame(self, text = "Filter für die Dateisuche")
        self.frm.grid(column = 0, row = 0, padx = 5, pady = 5, sticky = "nsew")

        self.addMenus()
        self.addWidgets()

        if os.path.isfile(DEFAULT_INI_FILE):
            self.onOpenDefault()

        else:
            # Das aktuelle Verzeichnis als Startverzeichnis auswählen
            self.startDir = os.path.abspath('.')
            self.backDir  = os.path.abspath('.')
            self.entry[0].set(self.startDir)
            self.entry[1].set(self.backDir)

        self.tmpFile  = DEFAULT_TMP_FILE

        self.mainloop()
        self.destroy()


    def check_backDir(self):
        """
        Prüft, ob der Eintrag im Eingabefeld für das Backup-Verzeichnis
        gültig ist (es können manuelle Eingaben fehlerhaft sein).
        Wenn nicht, wird ein Dialog zur Auswahl eines Verzeichnis geöffnet.
        """
        self.backDir = os.path.normcase(self.entry[1].get())
        self.backDir = self.cut_backslash(self.backDir)

        if not os.path.isdir(self.backDir):
            dir = filedialog.askdirectory(
                title = 'Ungültiges Verzeichnis {}\n'.format(self.backDir.upper()) +\
                    'Ein Verzeichnis zum Speichern der Zip-Datei auswählen :')
            if dir:
                dir = os.path.normcase(dir)
            else:
                tk.messagebox.showwarning(self.appName, \
                    "Kein Verzeichnis ausgewählt !\n" + \
                    "Das aktuelle Verzeichnis wird als Backup-Verzeichnis gewählt!")
                dir = os.path.abspath('.')
            self.backDir = dir

        self.entry[1].set(self.backDir)


    def cut_backslash(self, dir):
        """
        Entfernt den '\' bei den Pfadbezeichnungen,
        nicht aber bei Laufwerksbezeichnungen
        """
        # Laufwerksbezeichnung ohne Backslash
        if len(dir) == 2 and dir[1] == ':':
            dir = dir + '\\'

        # angehängte '\' entfernen, auch wenn es mehrere sind.
        elif dir.rfind('\\') == len(dir) - 1:
            while True:
                # Bei Laufwerksbezeichnung den '\' belassen
                if len(dir) < 4:
                    break
                if dir.rfind('\\') == len(dir) - 1:
                    dir = dir[:-1]
                else:
                    break
        return dir


    def addMenus(self):
          # so geht's auch, ist aber etwas unübersichtlicher
##        m = Tk.Menu(self)
##        self['menu'] = m
##        datei = Tk.Menu(master = m)
##        datei.add_command(label = "Als Default speichern", command = self._save, underline = 4)
##        datei.add_separator()
##        datei.add_command(label = "Einstellungen speichern unter", command = self._save_as, underline = 14)
##        datei.add_command(label = "Einstellungen laden", command = self._open_file, underline = 14)
##        datei.add_separator()
##        datei.add_command(label = "Beenden", command = self.quit, underline = 0)
##        m.add_cascade(label = "Datei", menu = datei, underline = 0)

        # Mark Lutz, Programming Python, S. 605
        menus = (
                ("Datei", 0,[
                    ("Als Default speichern",          4, self.onSave),
                    ("---"),
                    ("Einstellungen speichern unter", 14, self.onSaveAs),
                    ("Einstellungen laden",           14, self.onOpenFile),
                    ("---"),
                    ("Beenden",                        0, self.onQuit)]),
                    )

        menubar = tk.Menu(self)
        #self['menu'] = menubar
        self.config(menu = menubar)
        for name, underline, items in menus:
            pulldown = tk.Menu(menubar, tearoff=False)
            self.addMenuItems(pulldown, items)
            menubar.add_cascade(label = name, underline = underline, menu = pulldown)


    def addMenuItems(self, menu, items):
        for item in items:
            if item == "---":
                menu.add_separator()
            elif type(item) != list:
                menu.add_command(label = item[0],
                                 underline = item[1],
                                 command = item[2])

    def addWidgets(self):
        #    Textfeld, EingabefeldBreite, Checkbutton->negiert, Checkbutton->Aktiv
        items = (
            ("Die Suche beginnt im Verzeichnis :",            39,  False, False),
            ("Backups speichern im Verzeichnis :",            39,  False, False),
            ("Include Pfade mit SubStrings :",                39,  True,  True),
            ("Include DateiErweiterungen :",                  39,  True,  True),
            ("Include SubStrings im Dateinamen :",            39,  True,  True),
            ("Include Dateien kleiner/gleich -> ([b],k,m) :", 17,  False, True),
            ("Include Dateien  größer/gleich -> ([b],k,m) :", 17,  False, True),
            ("Include Dateien  älter als -> (m,[h],d,y) :",   17,  False, True),
            ("Include Dateien jünger als -> (m,[h],d,y) :",   17,  False, True),
            ("Das Suchergebnis mit Script bearbeiten :"       ,0,  False, False),)

        self.entry   = []
        self.negiert = []
        self.activ   = []
        row = 0

        for i in items:
            self.entry.append(tk.StringVar())
            self.negiert.append(tk.IntVar())
            self.activ.append(tk.IntVar())

            tk.Label(self.frm, text = i[0], width = 40, anchor = "e").\
                grid(column = 0, row = row, padx = 5, pady = 5, sticky = "nse")
            if i[1]:
                tk.Entry(self.frm, textvariable = self.entry[row], width = i[1]).\
                    grid(column = 1, row = row, padx = 5, pady = 5, sticky = "nsw")
            if i[2]:
                tk.Checkbutton(self.frm, variable = self.negiert[row], text = "Exclude").\
                            grid(column = 2, row = row, padx = 5, pady = 5, sticky = "nsw")
            if i[3]:
                tk.Checkbutton(self.frm, variable = self.activ[row], text = "aktiv").\
                            grid(column = 3, row = row, padx = 5, pady = 5, sticky = "nsw")
            row += 1

        # Hier wird das Dictionary für die Scripts aus dem Modul 'scripts' übernommen.
        self.combo_dict = scripts.scripts_dict

        self.combo_values = []
        for key in self.combo_dict.keys():
            self.combo_values.append(key)
        self.combo_values.sort()

        self.combo_var = self.entry[len(self.activ) - 1]          # unbenutzte tk.StringVar() nutzen

        self.combo_var.set('Dateien zippen')    # Vorgabewert
        self.combo = ttk.Combobox(self.frm, state = 'readonly', \
                values = self.combo_values, width = 17, \
                textvariable = self.combo_var)
        self.combo.grid(column = 1, row = len(self.activ) - 1, padx = 5, pady = 5, sticky = "nsw")


        # zum Abschluss die Buttons definieren
        tk.Button(self.frm, text = "...", width = 10, command = self.onFileDir).\
                grid(column = 3, row = 0, padx = 5, pady = 5, sticky = "nsew")

        tk.Button(self.frm, text = "...", width = 10, command = self.onBackDir).\
                grid(column = 3, row = 1, padx = 5, pady = 5, sticky = "nsew")

        tk.Button(self.frm, text = "Filter prüfen", width = 17, command = self.onCheck).\
                grid(column = 1, row = row, padx = 5, pady = 5, sticky = "nsw")

        tk.Button(self.frm, text = "Dateien zählen", width = 17, command = self.onFileCnt).\
                grid(column = 1, row = row, padx = 5, pady = 5, sticky = "nse")

        tk.Button(self.frm, text = "Script ausführen", command = self.onScript).\
                grid(column = 2, columnspan = 2, row = row - 1, padx = 5, pady = 5, sticky = "nsew")

        tk.Button(self.frm, text = "Dateien filtern", command = self.onFileList).\
                grid(column = 2, columnspan = 2, row = row, padx = 5, pady = 5, sticky = "nsew")


    def onFileDir(self):
        dir = filedialog.askdirectory(title = "Das Startverzeichnis "+\
                                        "für die Dateisuche auswählen :")

        if dir:
            dir = os.path.normcase(dir)
            self.entry[0].set(dir)
            self.startDir = dir


    def onBackDir(self):
        dir = filedialog.askdirectory(title = "Das Verzeichnis "+\
                                        "zum Speichern der Backups auswählen :")
        if dir:
            dir = os.path.normcase(dir)
            self.entry[1].set(dir)
            self.backDir = dir


    def onQuit(self,e = None):
        """
        Das Programm beenden, aber vorher nachfragen
        wenn e übergeben wird, dann wurde die Taste Escape gedrückt,
        hier wird beendet ohne die lästige Nachfrage.
        """
        if e is None:
            if messagebox.askokcancel(self.appName, "Das Programm wirklich beenden ?"):
                self.quit()
        else:
            self.quit()


    def onSaveAs(self):
        """ Die Einstellungen unter einem neuen Namen speichern """
        filename = filedialog.asksaveasfilename(defaultextension = '.wlk',\
                    title = "Einstellungen speichern",\
                    initialfile = "default",\
                    filetypes =[("WLK", "*.wlk"),
                                ("alle Dateien", "*.*")] )
        if filename:
            self.onSave(filename = filename)


    def onSave(self, filename = DEFAULT_INI_FILE):
        """ Schreibt die aktuellen Einstellungen in eine Datei """
        values = []
        for items in self.entry, self.negiert, self.activ:
            for item in items:
                values.append(item.get())
        f = open(filename, "wb")
        pickle.dump(values, f)
        f.close()


    def onOpenFile(self):
        """ Fragt nach einem Dateinamen vor dem Einlesen der Einstellungen """
        filename = filedialog.askopenfilename(defaultextension = '.wlk',\
                    title = "Einstellungen laden",\
                    initialfile = "default",\
                    filetypes =[("WLK", "*.wlk"),
                                ("*", "*.*")] )
        if filename:
            self.onOpenDefault(file = filename)


    def onOpenDefault(self, file = None):
        """ Öffnet die Default-Datei - oder eine andere """

        if file is None:
            if os.path.isfile(DEFAULT_INI_FILE):
                f = open(DEFAULT_INI_FILE, "rb")
        else:
            f = open(file, 'rb')

        idx = 0
        try:
            values = pickle.load(f)
            for items in self.entry, self.negiert, self.activ:
                for item in items:
                    item.set(values[idx])
                    idx += 1

            self.combo_var.set(self.entry[len(self.activ)-1].get())
            self.startDir = self.entry[0].get()
            self.backDir  = self.entry[1].get()

        except:
            messagebox.showwarning(self.appName, \
                    "Fehler beim Einlesen von Einstellung\n"
                    + "aus der Datei {}!".format(f))

# -------------------------------------------------------------------------
# Die weitere Aktion wird bestimmt durch den jeweils gedrückten Button

    def onScript(self):
        # es wird das bestehende tmp-File mit einem Script bearbeitet.
        if os.path.isfile(self.tmpFile):
            self.myfunction = self.combo_dict.get(self.combo_var.get())
            if self.myfunction is not None:
                # Ein Verweis auf self wird mitgegeben,
                # damit kann auf die Variablen von hier zugegriffen werden
                # sofern deren Name bekannt ist.
                self.myfunction(self)
        else:
            messagebox.showwarning(self.appName, \
                        "Es ist keine tmp-Datei vorhanden\n"\
                        + "Zuerst eine Suche ausführen !")


    def onFileCnt(self):
        """
        Zählt nur die zu den Filtereinstellungen passenden Verzeichnisse
        und Dateien etc durch.
        """
        self.onCheck(_checkfilter = False, _countfiles = True, _writelist = False)


    def onFileList(self):
        """
        Holt die Dateinamen aller zutreffenden Dateien und gibt sie als
        Liste im Editor aus bzw. schreibt sie in eine Datei.
        """
        self.onCheck(_checkfilter = False, _countfiles = False, _writelist = True)


    def onCheck(self, _checkfilter = True, _countfiles = False, _writelist = False):
        """
        Wertet nur die Filterbedingungen aus und gibt sie im Editro aus,
        macht weiter nichts.
        """
        self.checkfilter = _checkfilter
        self.countfiles  = _countfiles
        self.writelist   = _writelist

        errors = 0
        self.systime = int(time.time())

        self.f = StringIO()

# --- StartDir
        self.startDir = self.entry[0].get().strip()
        self.f.write("StartDir : ")
        if self.startDir:
            self.startDir = os.path.normcase(self.startDir)
            self.startDir = self.cut_backslash(self.startDir)

            self.entry[0].set(self.startDir)
            self.f.write(self.startDir)
            if os.path.isdir(self.startDir):
                self.f.write(" -> Das Startverzeichnis ist vorhanden !\n")
            else:
                self.f.write(" -> Das Startverzeichnis wurde nicht gefunden !\n")
                self.startDir = None
                errors += 1
        else:
            self.startDir = None
            self.f.write(" -> kein StartDir angegeben !\n")
            errors += 1

# ---- backDir
        self.backDir = self.entry[1].get().strip()
        self.f.write("BackDir  : ")
        if self.backDir:
            self.backDir = os.path.normcase(self.backDir)
            self.backDir = self.cut_backslash(self.backDir)

            self.entry[1].set(self.backDir)
            self.f.write(self.backDir)
            if os.path.isdir(self.backDir):
                self.f.write(" -> Das Backupverzeichnis ist vorhanden !\n\n")
            else:
                self.f.write(" -> Das Backupverzeichnis wurde nicht gefunden !\n\n")
                self.backDir = None
                errors += 1
        else:
            self.backDir = None
            self.f.write(" -> kein StartDir angegeben !\n\n")
            errors += 1

        filter_off = 0

# pathstring -[ -2- ] ----------------------------------------------------------
        if self.activ[2].get():
            tmp = self.entry[2].get().strip().split(',')
            # Leerzeichen und leere Zeichenketten entfernen
            pathstr = []
            for item in tmp:
                item = item.strip()
                if item:
                    pathstr.append(item)

            if not pathstr:
                pathstr = None

            else:
                if self.negiert[2].get(): txt = "Exclude"
                else: txt = "Include"
                self.f.write("{}  : Paths mit SubStrings: {}\n".\
                            format(txt, pathstr))
        else:
            pathstr = None
            filter_off += 1

# Extension -[ -3- ] -----------------------------------------------------------
        if self.activ[3].get():
            tmp = self.entry[3].get().split(',')
            extstr = []

            # keine leeren Einträge in die Liste übernehmen
            # um eine Datei ohne Dateierweiterung zu finden
            # muss  ein '.' eingegeben werden
            # walker sieht bei leeren Extensions nach,
            # ob in der Liste '.' vorhanden ist
            for ext in tmp:
                 ext = ext.strip()
                 if ext:
                    extstr.append(ext)

            if not extstr:
                extstr = None

            else:
                if self.negiert[3].get(): txt = "Exclude"
                else: txt = "Include"

                # den Punkt vor der Dateierweiterung ergänzen, falls er fehlen sollte.
                if extstr is not None:
                    for i in range(len(extstr)):
                        if not extstr[i].startswith('.'):
                            extstr[i] = ('.{}'.format(extstr[i]))

                self.f.write("{}  : FileExtensions: {}\n".format(txt, extstr))
        else:
            extstr = None
            filter_off += 1

# Filename - [ -4- ] -----------------------------------------------------------
        if self.activ[4].get():
            tmp = self.entry[4].get().split(',')
            namestr = []
            for item in tmp:
                item = item.strip()
                if item:
                    namestr.append(item)

            if not namestr:
                namestr = None

            else:
                if self.negiert[4].get(): txt = "Exclude"
                else: txt = "Include"

                self.f.write("{}  : FileNames with Substrings: {}\n".\
                            format(txt, namestr))
        else:
            namestr = None
            filter_off += 1

# Filesize --- [ -5-6- ] -------------------------------------------------------
        if self.activ[5].get():
            size_min = check_size_entry(self.entry[5].get())
            if size_min is None:
                size_min = 0
                self.f.write("         : MinFileSize - error ----> '{}'\n".format(
                            self.entry[5].get()))
                errors += 1
            else:
                self.f.write("Include  : Files kleiner/gleich {} Bytes\n".\
                                format(size_min))
        else:
            size_min = None
            filter_off += 1

        if self.activ[6].get():
            size_max = check_size_entry(self.entry[6].get())
            if size_max is None:
                size_max = 0
                self.f.write("    : MaxFileSize - error ----> '{}'\n".\
                                format(self.entry[5].get()))
                errors += 1

            else:
                self.f.write("Include  : Files  größer/gleich {} Bytes\n".\
                                format(size_max))
        else:
            size_max = None
            filter_off += 1

        if self.activ[5].get() and self.activ[6].get():
            if size_min < size_max:
                errors += 1
                self.f.write("\nMinimale Filegröße {} kleiner als Maximale Filegröße {}\n\n".
                                format(size_min, size_max))
                messagebox.showwarning(self.appName, "Fehler !\n\nEs wird nach Dateien mit \n"+
                "weniger als {} Bytes und \nmehr als {} Bytes gesucht !".\
                                format(size_min, size_max))

# Filetime --- [ -7-8- ] -------------------------------------------------------
        if self.activ[7].get():

            time_min = check_time_entry(self.entry[7].get() )
            if time_min is not None:
                time_min = self.systime - time_min
                t = time.localtime(time_min)
                date_min = "{:02d}.{:02d}.{} {:02}:{:02}:{:02}".\
                            format(t[2], t[1], t[0], t[3], t[4], t[5])
                self.f.write("Include  : Files älter  als {}\n".\
                            format(date_min))
            else:
                time_min = 0
                date_min = "error"
                self.f.write("Include  :  MinFileTime - error ----> '{}'\n".\
                            format(self.time_min.get()))
                errors += 1

        else:
            time_min = None
            filter_off += 1

        if self.activ[8].get():
            time_max = check_time_entry(self.entry[8].get() )
            if time_max is not None:
                time_max = self.systime - time_max
                t = time.localtime(time_max)
                date_max = "{:02d}.{:02d}.{} {:02}:{:02}:{:02}".\
                            format(t[2], t[1], t[0], t[3], t[4], t[5])
                self.f.write("Include  : Files jünger als {}\n".\
                            format(date_max))
            else:
                time_max = 0
                date_max = "error"
                self.f.write("Include  : MaxFileTime - error ----> '{}'\n".\
                            format(self.time_max))
                errors += 1
        else:
            time_max = None
            filter_off += 1

        if self.activ[7].get() and self.activ[8].get():
            if time_max >= time_min:
                errors += 1
                self.f.write("\nMinimale FileTime {} größer/gleich Maximale FileTime {}\n\n".\
                            format(date_min, date_max))
                messagebox.showwarning(self.appName, "Fehler !\n\nEs wird nach Dateien \n"+
                "jünger als {} Sekunden und \nälter als als {} Sekunden gesucht !".\
                            format(time_min, time_max))

# Bearbeiten --- [ -9- ] -------------------------------------------------------
        if self.activ[9].get():
            self.myfunction = self.combo_dict.get(self.combo_var.get())
            self.f.write("Active    : Bearbeiten -> '{}' - Funktion: '{}'\n".format(
                    self.combo_var.get(), self.myfunction))

# Auf Fehler auswerten ---------------------------------------------------------
        if errors:
            self.f.write("\n-> {} Fehler im StartDir oder in den Eingabezeilen\n".\
                                                    format(errors))

# ------------------------------------------------------------------------------
        # Prüfen, ob überhaupt irgendein Filter aktiviert wurde
        if filter_off > 6:
            self.f.write("\nEs ist kein einiger Filter aktiviert !")
            errors = 1

        # wenn Fehler aufgetreten sind oder nur der Filter angezeigt werden soll,
        # dann nur den Editor aufrufen - mehr nicht
        if errors or self.checkfilter:
            Editor(self, file = self.f, height = 20)
            return

        elif self.countfiles:
            # wenn nur gefiltert und das Ergebnis angezeigt wird - ohne nachfolgend
            # Bearbeitung
            self.viewer = FileCounter(master = self, callback = self.viewerClose,\
                            title = self.appName + " - Verzeichnisse und Dateien zählen")
            self.visitfiles = False
            self.f.close()
            self.f = None
            self.queue = queue.Queue()

        elif self.writelist:
            # das StringIO schließen und eine Datei öffnen
            self.f.close()
            self.f = open(self.tmpFile, 'w', encoding = "utf-8")
            self.f.write('+ {} + DIESE ZEILE NICHT EDITIEREN !\n'.format(self.startDir))
            self.viewer = SimpleBarGraphWindow(master = self, \
                        callback = self.viewerClose, title = self.appName)
            self.visitfiles = True
            self.queue = queue.Queue()

        #if not errors and (self.countfiles or self.writelist):
        if not errors:
            self.walker = FileVisitor(
                        path_negiert   = self.negiert[2].get(),
                        path_include   = pathstr,
                        ext_negiert    = self.negiert[3].get(),
                        ext_include    = extstr,
                        name_negiert   = self.negiert[4].get(),
                        name_include   = namestr,
                        size_min       = size_min,
                        size_max       = size_max,
                        time_min       = time_min,
                        time_max       = time_max,
                        startDir       = self.startDir,
                        f              = self.f,
                        reset          = True,
                        queue          = self.queue,
                        visitfiles     = self.visitfiles,)

            # den Hintergrundthread zur Dateisuche starten
            self.walker.setDaemon(True)
            self.walker.start()
            self.after(150, self.readQueue)


    def readQueue(self):
        """
        Gibt die Daten aus der Queue weiter an das anzeigende ToplevelWindow
        Allerdings wird diese Aktualisierung nur in größeren Zeitabständen
        ausgeführt, alle zwischendrin eingegangenen Einträge werden
        ausgelesen, aber ignoriert.
        """

        input = ""
        self.queue_tmp = ""
        queue_continue = True
        try:
            # liest solange die Queue aus, bis keine Daten mehr anstehen,
            # der letzte Datensatz steht dann in self.queue_tmp
            # und wird von except ausgewertet.
            while True:
                input = self.queue.get(block = False)
                # None ist das Zeichen für den Abschluss des Threads
                if input is None:
                    self.viewer.showStatus(self.queue_tmp)
                    queue_continue = False
                # der letzte Datensatz wird zwischengespeichert
                else:
                    self.queue_tmp = input

        except queue.Empty:
            # None ist das Zeichen für den Abschluss des Threads
            if input is not None:
                self.viewer.showStatus(input)

            else:
                self.viewer.showStatus(self.queue_tmp)
                queue_continue = False

        if queue_continue:
            self.after(150, self.readQueue)
        # das Einlesen der Queue wird beendet
        else:
            self.showResult()


    def showResult(self):
        # Wenn eine Dateiliste aufgezeichnet wurde, dann
        # den Bargraph schießen und den Editor aufrufen
        if self.visitfiles:
            self.viewer.closeWindow()
            self.viewer = None

        # wenn eine Liste angelegt wurde, die nur angezeigt werden soll
        if self.writelist:
            self.f.close()
            Editor(self, file = self.tmpFile)
            self.focus_force()


    def viewerClose(self):
        """
        Wird ausgeführt, wenn der "Beenden"-Button im Toplevel 'Viewer' gedrückt
        wurde: nun muss der Filewalker und der Viewer beendet werden
        """
        self.walker.killThread()  # beendet auch die Queue
        self.viewer.killWindow()



if __name__ == '__main__':
    gui()


